package xctx

import (
	"context"
	xsf "xmall/pkg/utils/xsnowflake"

	"github.com/gin-gonic/gin"
	"github.com/spf13/cast"
	"go.uber.org/zap"
)

var (
	GinContextKey = "gin_context"

	// ContextRequestIDKey request id for context
	ContextRequestIDKey = "traceID"
	KeyConfigFile       = "config_file"
	KeyConfigCenter     = "config_center"

	// HeaderXRequestIDKey header request id key
	HeaderXRequestIDKey = "X-Request-ID"

	KeyTid    = "tid"
	KeyIP     = "ip"
	KeyUID    = "uid"
	KeyShopID = "shopID"
	KeyUName  = "uname"
	KeyToken  = "token"
	KeyUser   = "user"
)

// GCtxRequestID get request id from gin.Context
func GCtxRequestID(c *gin.Context) string {
	if v, isExist := c.Get(ContextRequestIDKey); isExist {
		if requestID, ok := v.(string); ok {
			return requestID
		}
	}
	return ""
}

// GCtxRequestIDField get request id field from gin.Context
func GCtxRequestIDField(c *gin.Context) zap.Field {
	return zap.String(ContextRequestIDKey, GCtxRequestID(c))
}

// CtxRequestID get request id from context.Context
func CtxRequestID(c context.Context) string {
	v := c.Value(ContextRequestIDKey)
	if str, ok := v.(string); ok {
		return str
	}
	return ""
}

// CtxXRequestID get request id from context.Context
func CtxXRequestID(c context.Context) string {
	v := c.Value(HeaderXRequestIDKey)
	if str, ok := v.(string); ok {
		return str
	}
	return ""
}

func CtxGetGinCtx(c context.Context) *gin.Context {
	if str, ok := c.Value(GinContextKey).(*gin.Context); ok {
		return str
	}
	return nil
}

// CtxTraceIDField get request id field from context.Context
func CtxTraceIDField(c context.Context) zap.Field {
	return zap.String(ContextRequestIDKey, CtxRequestID(c))
}

func GetGinUserID(c *gin.Context) uint64 {
	uid, exists := c.Get(KeyUID)
	if exists {
		return cast.ToUint64(uid)
	}
	return 0
}

func GetGinShopID(c *gin.Context) uint64 {
	uid, exists := c.Get(KeyShopID)
	if exists {
		return cast.ToUint64(uid)
	}
	return 0
}

func GetCtxUserID(c context.Context) uint64 {
	uid := c.Value(KeyUID)
	if uid != nil {
		return cast.ToUint64(uid)
	}
	return 0
}

func GetCtxShopID(c context.Context) uint64 {
	uid := c.Value(KeyShopID)
	if uid != nil {
		return cast.ToUint64(uid)
	}
	return 0
}
func GetCtxString(c context.Context, key string) string {
	val := c.Value(key)
	if val != nil {
		return cast.ToString(val)
	}
	return ""
}

func GetCtxUserToken(c context.Context) string {
	token := c.Value(KeyToken)
	if token != nil {
		return cast.ToString(token)
	}
	return ""
}

func GetGinCtxTid(c *gin.Context) string {
	tid := c.GetString(KeyTid)
	if tid == "" {
		tid = GenerateTid()
		c.Set(KeyTid, tid)
	}
	return tid
}

func GenerateTid() string {
	return xsf.Generate().String()
}

func GetCtxTid(ctx context.Context) string {
	tid := ""
	tidVal := ctx.Value(KeyTid)
	if tidVal != nil {
		if str, ok := tidVal.(string); ok {
			tid = str
		}
	}
	if tid == "" {
		tid = GenerateTid()
	}
	return tid
}

func GetIP(ctx context.Context) string {
	ip := ""
	ipVal := ctx.Value(KeyIP)
	if ipVal != nil {
		if str, ok := ipVal.(string); ok {
			ip = str
		}
	}
	return ip
}

func WrapCtx(c *gin.Context) context.Context {
	ctx := context.WithValue(c.Request.Context(), ContextRequestIDKey, c.GetString(ContextRequestIDKey)) //nolint
	for k, v := range c.Keys {
		ctx = context.WithValue(ctx, k, v) //nolint
	}
	ctx = context.WithValue(ctx, GinContextKey, c) //nolint
	return ctx
}
